﻿using System;
using System.Collections.Generic;
using System.DirectoryServices;
using System.Globalization;
using ActiveDs;
using MM.Monitor.Client;
using PaulCsiki.SessionControl.Enums;
using PaulCsiki.SessionControl.Types;

namespace PaulCsiki.SessionControl
{
    public class SessionControlPlugin : ClientPlugin
    {
        private Dictionary<int, Session> SessionCache = new Dictionary<int, Session>();
        private Dictionary<int, OpenedFile> OpenedFilesCache = new Dictionary<int, OpenedFile>();
        private static int SessionKey = 0;
        private static int OpenedFilesKey = 0;

        public override string GetPluginName()
        {
            return "Session Control";
        }

        public override string GetPluginDescription()
        {
            return "Plugin used to manage and control network sessions.";
        }

        public override MM.Monitor.Client.Version GetMinimumRequiredAgentVersion()
        {
            return new MM.Monitor.Client.Version(2, 9, 3);
        }

        public override Groups GetAdditionalComputerDetails()
        {
            Groups container = new Groups();
            Group pages = new Group("Session Control");
            pages.Items.Add(new PageItem(GeneratePageID(PageID.LIST_SESSIONS, "9999"), "List Sessions"));
            pages.Items.Add(new PageItem(GeneratePageID(PageID.LIST_OPENED_FILES, "9999"), "List Opened Files"));
            container.Add(pages);
            return container;
        }

        public override Groups GetPageDetails(int pageId)
        {
            PageID pid = (PageID)Int32.Parse(pageId.ToString().Remove(2));
            string id = pageId.ToString().Remove(0, 2);
            Groups container = new Groups();
            string version = Environment.OSVersion.Version.ToString(3);
            Trace(String.Format("PageDetails - PID: {0} ID: {1} OSVersion: {2}", (int)pid, id, version));
            if (version != "5.2.3790" && version != "6.1.7601" && version != "6.1.7600")
            {
                Group error = new Group("Error");
                error.Items.Add(new SimpleItem("OS not supported", "Error"));
                container.Add(error);
                return container;
            }
            switch (pid)
            {
                case PageID.LIST_SESSIONS:
                    Group session_list = new Group("Session List");
                    var list_sessions = GetActiveSessions();
                    foreach (var s in list_sessions)
                        session_list.Items.Add(new PageItem(GeneratePageID(PageID.SESSION_DETAILS, AddToSessionCache(s).ToString()), s.Username, s.IP));
                    container.Add(session_list);
                    break;
                case PageID.LIST_OPENED_FILES:
                    Group file_list = new Group("Opened Files List");
                    List<OpenedFile> list_files;
                    if (id != null && id != string.Empty && id != "9999")
                    {
                        Session current = SessionCache[Int32.Parse(id)];
                        Trace("PageDetails - Opened Files List got username: " + current.Username);
                        list_files = GetOpenedFiles(current.Username);
                    }
                    else
                        list_files = GetOpenedFiles();
                    foreach (var of in list_files)
                        file_list.Items.Add(new PageItem(GeneratePageID(PageID.OPENED_FILE_DETAILS, AddToOpenedFilesCache(of).ToString()), of.Path, of.LockCount.ToString()));
                    container.Add(file_list);
                    break;
                case PageID.SESSION_DETAILS:
                    Group session_details = new Group("Session Details");
                    Session current_session = SessionCache[Int32.Parse(id)];
                    session_details.Items.Add(new SimpleItem("Username", current_session.Username));
                    session_details.Items.Add(new SimpleItem("IP", current_session.IP));
                    Group session_commands = new Group("Actions");
                    session_commands.Items.Add(new PageItem(GeneratePageID(PageID.LIST_OPENED_FILES, current_session.ID.ToString()), "Opened Files"));
                    container.Add(session_details);
                    container.Add(session_commands);
                    break;
                case PageID.OPENED_FILE_DETAILS:
                    Group file_details = new Group("Opened File Details");
                    OpenedFile current_file = OpenedFilesCache[Int32.Parse(id)];
                    file_details.Items.Add(new SimpleItem("Username", current_file.Username));
                    file_details.Items.Add(new SimpleItem("Path", current_file.Path));
                    file_details.Items.Add(new SimpleItem("Lock Count", current_file.LockCount.ToString()));
                    container.Add(file_details);
                    break;
            }
            return container;
        }

        private int AddToSessionCache(Session value)
        {
            if (SessionCache.Count == 9999)
            {
                SessionKey = 0;
                Session.ResetID();
            }
            SessionKey++;
            SessionCache.Add(SessionKey, value);
            return SessionKey;
        }

        private int AddToOpenedFilesCache(OpenedFile value)
        {
            if (OpenedFilesKey == 9999)
            {
                OpenedFilesKey = 0;
                OpenedFile.ResetID();
            }
            OpenedFilesKey++;
            OpenedFilesCache.Add(OpenedFilesKey, value);
            return OpenedFilesKey;
        }

        private static List<Session> GetActiveSessions()
        {
            DirectoryEntry container = new DirectoryEntry(String.Format("WinNT://{0}/LanmanServer", Environment.MachineName));
            IADsFileServiceOperations fso = container.NativeObject as IADsFileServiceOperations;
            List<Session> sessionList = new List<Session>();
            TextInfo myTI = new CultureInfo("en-US", false).TextInfo;
            if (fso != null)
                foreach (IADsSession sess in fso.Sessions())
                {
                    Session newSession = new Session(myTI.ToTitleCase(sess.User), sess.Computer);
                    sessionList.Add(newSession);
                }
            container.Close();
            container.Dispose();
            return sessionList;
        }

        private static List<OpenedFile> GetOpenedFiles()
        {
            DirectoryEntry container = new DirectoryEntry(String.Format("WinNT://{0}/LanmanServer", Environment.MachineName));
            IADsFileServiceOperations fso = container.NativeObject as IADsFileServiceOperations;
            IADsCollection resources = fso.Resources() as IADsCollection;
            List<OpenedFile> fileList = new List<OpenedFile>();
            TextInfo myTI = new CultureInfo("en-US", false).TextInfo;
            if (fso != null)
                foreach (IADsResource resource in resources)
                {
                    OpenedFile newFile = new OpenedFile(myTI.ToTitleCase(resource.User), resource.Path, resource.LockCount);
                    fileList.Add(newFile);
                }
            container.Close();
            container.Dispose();
            return fileList;
        }

        private List<OpenedFile> GetOpenedFiles(string Username)
        {
            DirectoryEntry container = new DirectoryEntry(String.Format("WinNT://{0}/LanmanServer", Environment.MachineName));
            IADsFileServiceOperations fso = container.NativeObject as IADsFileServiceOperations;
            IADsCollection resources = fso.Resources() as IADsCollection;
            List<OpenedFile> fileList = new List<OpenedFile>();
            TextInfo myTI = new CultureInfo("en-US", false).TextInfo;
            if (fso != null)
                foreach (IADsResource resource in resources)
                {
                    if (resource.User.ToLower() == Username.ToLower())
                    {
                        OpenedFile newFile = new OpenedFile(myTI.ToTitleCase(resource.User), resource.Path, resource.LockCount);
                        fileList.Add(newFile);
                    }
                    else
                        Trace(String.Format("GetOpenedFiles not matching: {0} with: {1}", resource.User, Username.ToUpper()));
                }
            container.Close();
            container.Dispose();
            return fileList;
        }

        private static int GeneratePageID(PageID pid, string itemID)
        {
            while (itemID.Length < 4)
            {
                itemID = itemID.Insert(0, "0");
            }

            return Int32.Parse(String.Format("{0}{1}", (int)pid, itemID));
        }
    }
}